<?php
declare(strict_types = 1);

/**
 * Gestion des catégories.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Category
{
	/**
	 * Séparateur des catégories parentes pour
	 * la colonne "cat_parents" de la table "categories".
	 *
	 * @var string
	 */
	const PARENTS_SEPARATOR = '.';



	/**
	 * Création d'une catégorie.
	 *
	 * @param string $cat_type
	 *   Type de la catégorie ('category' ou 'album').
	 * @param string $cat_name
	 *   Nom de la catégorie.
	 * @param string $cat_desc
	 *   Description de la catégorie.
	 * @param int $cat_status
	 *   État de la catégorie.
	 * @param int $parent_id
	 *   Identifiant de la catégorie parente.
	 * @param int $user_id
	 *   Identifiant de l'utilisateur propriétaire.
	 *
	 * @return bool
	 */
	public static function create(string $cat_type, string $cat_name, string $cat_desc = '',
	int $cat_status = 0, int $parent_id = 1, int $user_id = 1): bool
	{
		// Quelques vérifications basiques.
		if (!in_array($cat_status, [0, 1]) || $parent_id < 1 || $user_id < 1)
		{
			return FALSE;
		}

		// Nettoyage et vérification de la longueur du nom.
		$cat_name = Utility::trimAll($cat_name);
		if (strlen($cat_name) < 1)
		{
			return FALSE;
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return FALSE;
		}

		// Récupération des informations utiles de la catégorie parente.
		$sql = "SELECT cat_id,
					   password_id,
					   cat_parents,
					   cat_path
				  FROM {categories} WHERE cat_id = ?";
		if (!DB::execute($sql, $parent_id))
		{
			goto fail;
		}
		$parent_infos = DB::fetchRow();
		if (!isset($parent_infos['cat_id']))
		{
			goto fail;
		}

		// On génère le nom de répertoire.
		$parent_path = CONF_ALBUMS_PATH . '/' . $parent_infos['cat_path'];
		$cat_dirname = App::getValidDirname($cat_name);
		$cat_dirname = File::getSecureFilename($cat_dirname, $parent_path);
		if (strlen($cat_dirname) < 1)
		{
			goto fail;
		}

		// On ajoute la nouvelle catégorie à la base de données.
		$sql = "INSERT INTO {categories} (
			user_id, thumb_id, password_id, parent_id, cat_parents, cat_path,
			cat_name, cat_url, cat_desc, cat_crtdt, cat_filemtime, cat_status
			) VALUES (
			:user_id, :thumb_id, :password_id, :parent_id, :cat_parents, :cat_path,
			:cat_name, :cat_url, :cat_desc, :cat_crtdt, :cat_filemtime, :cat_status
			)";
		$cat_parents = $parent_infos['cat_parents'];
		if ($parent_id > 1)
		{
			$cat_parents .= $parent_id . self::PARENTS_SEPARATOR;
		}
		$cat_path = ($parent_id == 1)
			? $cat_dirname
			: $parent_infos['cat_path'] . '/' . $cat_dirname;
		$datetime = date('Y-m-d H:i:s');
		$params =
		[
			'user_id' => $user_id,
			'thumb_id' => -1,
			'password_id' => $parent_infos['password_id'],
			'parent_id' => $parent_id,
			'cat_parents' => $cat_parents,
			'cat_path' => $cat_path,
			'cat_name' => $cat_name,
			'cat_url' => App::getURLName($cat_name),
			'cat_desc' => (Utility::trimAll($cat_desc) === '')
				? NULL
				: $cat_desc,
			'cat_crtdt' => $datetime,
			'cat_filemtime' => ($cat_type == 'album')
				? $datetime
				: NULL,
			'cat_status' => $cat_status
		];
		if (!DB::execute($sql, $params))
		{
			goto fail;
		}

		// Mise à jour de la colonne "cat_position".		
		$sql = "SELECT cat_id FROM {categories} WHERE cat_path = ?";
		if (!DB::execute($sql, $cat_path))
		{
			goto fail;
		}
		$cat_id = DB::fetchVal();
		$sql = "UPDATE {categories} SET cat_position = ? WHERE cat_id = ?";
		if (!DB::execute($sql, [$cat_id, $cat_id]))
		{
			goto fail;
		}

		// On met à jour le nombre de sous-catégories de la catégorie parente.
		if (!Parents::updateSubCats([$parent_id]))
		{
			goto fail;
		}

		// On update les catégories parentes du nombre d'albums.
		if ($cat_type == 'album')
		{
			$stats =
			[
				'albums' => 1,
				'comments' => 0,
				'favorites' => 0,
				'hits' => 0,
				'images' => 0,
				'rating' => 0,
				'size' => 0,
				'videos' => 0,
				'votes' => 0
			];
			if ($cat_status)
			{
				$cat_update = ['a' => $stats];
				$a_sign = '+';
				$d_sign = '';
			}
			else
			{
				$cat_update = ['d' => $stats];
				$a_sign = '';
				$d_sign = '+';
			}
			$parents_id = self::getParentsIdArray($cat_parents);
			if (!Parents::updateStats($cat_update, $a_sign, $d_sign, $parents_id))
			{
				goto fail;
			}
		}

		if (CONF_DEV_MODE && Maintenance::dbStats() !== 0)
		{
			trigger_error('Gallery stats error.', E_USER_WARNING);
			goto fail;
		}

		$realpath = $parent_path . '/' . $cat_dirname;

		// On tente de créer le répertoire.
		if (!File::mkdir($realpath))
		{
			goto fail;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			// On supprime le répertoire en cas d'échec de la transaction.
			if (is_dir($realpath))
			{
				File::rmdir($realpath);
			}
			goto fail;
		}

		return TRUE;

		fail:
		DB::rollbackTransaction();
		return FALSE;
	}

	/**
	 * Supprime des catégories, avec toutes leurs sous-catégories,
	 * albums, images, vidéos, tags, votes et commentaires qui leurs sont liés.
	 *
	 * @param array $categories_id
	 *   Identifiant des catégories.
	 *
	 * @return int
	 *   Retourne le nombre de catégories affectées
	 *   ou -1 en cas d'erreur.
	 */
	public static function delete(array $categories_id): int
	{
		if (!$categories_id)
		{
			return 0;
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return -1;
		}

		// Récupération des informations des catégories.
		if (!($categories_by_parent = self::_getCategoriesByParent($categories_id)))
		{
			goto nochange;
		}

		// On traite les catégories catégorie parente par catégorie parente.
		$update_parents_id = [];
		$categories_deleted = 0;
		foreach ($categories_by_parent as &$categories_infos)
		{
			// Suppression des catégories.
			$sql = "DELETE
					  FROM {categories}
					 WHERE cat_id IN (" . DB::inInt(array_keys($categories_infos)) . ")";
			if (!DB::execute($sql))
			{
				goto dbfail;
			}
			$categories_deleted += DB::rowCount();

			// On calcule les valeurs des colonnes
			// des catégories parentes à mettre à jour.
			$stats =
			[
				'albums' => 0,
				'comments' => 0,
				'favorites' => 0,
				'hits' => 0,
				'images' => 0,
				'rating' => 0,
				'size' => 0,
				'videos' => 0,
				'votes' => 0
			];
			$cat_update = ['a' => $stats, 'd' => $stats];
			foreach ($categories_infos as &$i)
			{
				// Nombre d'albums.
				if ($i['cat_filemtime'] === NULL)
				{
					$cat_update['a']['albums'] += $i['cat_a_albums'];
					$cat_update['d']['albums'] += $i['cat_d_albums'];
				}
				else
				{
					$status = ($i['cat_status']) ? 'a' : 'd';
					$cat_update[$status]['albums'] += 1;
				}

				// Autres stats.
				self::_parentsUpdateStats($cat_update['a'], $i, 'a');
				self::_parentsUpdateStats($cat_update['d'], $i, 'd');
			}

			// Mise à jour des statistiques des catégories parentes.
			$parents_id = self::getParentsIdArray(current($categories_infos)['cat_parents']);
			$update_parents_id += array_flip($parents_id);
			if (!Parents::updateStats($cat_update, '-', '-', $parents_id))
			{
				goto dbfail;
			}

			// On met à jour la date d'ajout du dernier fichier
			// et la vignette des catégories parentes.
			if (!Parents::updateInfos($parents_id))
			{
				goto dbfail;
			}
		}

		if ($categories_deleted < 1)
		{
			goto nochange;
		}

		// On met à jour le nombre de sous-catégories.
		if (!Parents::updateSubCats(array_keys($update_parents_id)))
		{
			goto dbfail;
		}

		if (CONF_DEV_MODE && Maintenance::dbStats() !== 0)
		{
			trigger_error('Gallery stats error.', E_USER_WARNING);
			goto dbfail;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			goto dbfail;
		}

		// On supprime les catégories sur le disque.
		foreach ($categories_by_parent as &$categories)
		{
			foreach ($categories as &$i)
			{
				$dir = CONF_ALBUMS_PATH . '/' . $i['cat_path'];
				if (!file_exists($dir))
				{
					continue;
				}
				if (!File::rmdir($dir))
				{
					return -1;
				}
			}
		}

		return $categories_deleted;

		dbfail:
		DB::rollbackTransaction();
		return -1;

		nochange:
		DB::rollbackTransaction();
		return 0;
	}

	/**
	 * Édite les informations de plusieurs catégories.
	 *
	 * @param array $categories_update
	 *   Informations de mise à jour.
	 *
	 * @return int
	 *   Retourne le nombre de catégories affectées
	 *   ou -1 en cas d'erreur.
	 */
	public static function edit(array &$categories_update): int
	{
		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return -1;
		}

		// Récupération des informations des catégories.
		$categories_id = DB::inInt(array_keys($categories_update));
		$sql = "SELECT * FROM {categories} WHERE cat_id IN ($categories_id)";
		if (!DB::execute($sql))
		{
			goto dbfail;
		}
		$categories_infos = DB::fetchAll('cat_id');
		if (!$categories_infos)
		{
			goto nochange;
		}
		$categories_id = array_keys($categories_infos);

		// Mise à jour des informations des catégories.
		$params = [];
		$rename = [];
		$order_by_params_album = self::getOrderByParams()['album'];
		$order_by_params_category = self::getOrderByParams()['category'];
		foreach ($categories_update as $id => &$update)
		{
			if (!array_key_exists($id, $categories_infos))
			{
				continue;
			}

			// On regroupe les critères de tri séparés.
			$album = $categories_infos[$id]['cat_filemtime'] !== NULL;
			$order_by_params = $categories_infos[$id]['cat_filemtime'] === NULL
				? $order_by_params_category
				: $order_by_params_album;
			$cat_orderby = [];
			foreach ([1, 2, 3] as $n)
			{
				if (array_key_exists('sortby_' . $n, $update)
				 && array_key_exists('orderby_' . $n, $update)
				 && isset($order_by_params[$update['sortby_' . $n]])
				 && in_array($update['orderby_' . $n], ['ASC', 'DESC']))
				{
					$cat_orderby[] = $update['sortby_' . $n] . ' ' . $update['orderby_' . $n];
				}
			}
			if (count($cat_orderby) === 3)
			{
				$cat_orderby = implode(', ', $cat_orderby);
				$update['cat_orderby'] = $cat_orderby;
			}

			// Colonnes modifiables.
			$cols =
			[
				'password_id',
				'cat_name',
				'cat_url',
				'cat_path',
				'cat_desc',
				'cat_orderby',
				'cat_commentable',
				'cat_creatable',
				'cat_downloadable',
				'cat_uploadable',
				'cat_votable'
			];
			$p = [];
			foreach ($cols as &$name)
			{
				$p[$name] = $categories_infos[$id][$name];
			}
			$p_temp = $p;

			foreach ($update as $col => &$value)
			{
				if (!array_key_exists($col, $p))
				{
					continue;
				}

				switch ($col)
				{
					// Mot de passe.
					case 'password_id' :
						$r = self::_changePassword($categories_infos[$id], $value);
						if ($r === FALSE)
						{
							goto dbfail;
						}
						if ($r['change'])
						{
							$p_temp[$col] = 0;
							$p[$col] = $r['id'];
						}
						if ($r['id'] !== NULL)
						{
							$value = '**********';
						}
						break;

					// Titre et nom d'URL.
					case 'cat_name' :
					case 'cat_url' :
						$val = Utility::trimAll((string) $value);
						$val = $col == 'cat_url' ? App::getURLName($val) : $val;
						$val = substr($val, 0, 255);
						if (strlen($val) > 0)
						{
							$p[$col] = $val;
						}
						break;

					// Description.
					case 'cat_desc' :
						$val = Utility::trimAll((string) $value);
						$val = substr($val, 0, 65535);
						$p[$col] = $val === '' ? NULL : $val;
						break;

					// Nom de répertoire.
					case 'cat_path' :
						$r = self::_changeDirname($categories_infos[$id], $value);
						if ($r === FALSE)
						{
							goto dbfail;
						}
						$p[$col] = $r[0];
						if (count($r) > 1)
						{
							$rename[$r[1]] = $r[2];
						}
						break;

					// Permissions.
					case 'cat_commentable' :
					case 'cat_creatable' :
					case 'cat_downloadable' :
					case 'cat_uploadable' :
					case 'cat_votable' :

						// On ne peut pas modifier la permission si
						// la permission parente est à 0.
						$parts = explode('_', $col);
						if ($update['parent_' . $parts[1]]
						&& ($col != 'cat_creatable' || $col == 'cat_creatable'
						&& $categories_infos[$id]['cat_filemtime'] === NULL))
						{
							$p[$col] = (string) (int) $value;
						}
						else
						{
							$p[$col] = $categories_infos[$id][$col];
						}
						break;

					// Critères de tri.
					case 'cat_orderby' :
						if (preg_match('`^([\sa-z_]{3,30}, ){3}$`i', $value . ', '))
						{
							$p[$col] = strstr($value, 'default') ? NULL : $value;
						}
						break;
				}
			}

			if ($p !== $p_temp)
			{
				$p['cat_id'] = $id;
				$params[] = $p;
			}
		}

		if (!$params)
		{
			goto nochange;
		}

		// Colonnes à mettre à jour.
		$columns = '';
		foreach (array_keys($params[0]) as &$col)
		{
			$columns .= $col . ' = :' . $col . ', ';
		}
		$columns = substr($columns, 0, -2);

		// On met à jour les catégories.
		$sql = "UPDATE {categories} SET $columns WHERE cat_id = :cat_id";
		if (!DB::execute($sql, $params))
		{
			goto dbfail;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			goto dbfail;
		}

		// On renomme les catégories.
		foreach ($rename as $old_path => &$new_path)
		{
			if (file_exists($old_path))
			{
				File::rename($old_path, $new_path);
			}
		}

		return count($params);

		dbfail:
		DB::rollbackTransaction();
		return -1;

		nochange:
		DB::rollbackTransaction();
		return 0;
	}

	/**
	 * Retourne quelques informations de toutes les catégories de la galerie.
	 * Cette méthode permet également de récupérer la fonction $makelist
	 * permettant de générer facilement une liste HTML des catégories.
	 *
	 * @param Closure $makelist
	 * @param Closure $get_url
	 *   Fonction permettant de créer un lien.
	 * @param int $category_id
	 *   Identifiant de la catégorie courante.
	 * @param string $perms
	 *   Code SQL contenant les restrictions d'accès aux catégories.
	 *
	 * @return bool|array
	 *   Retourne FALSE en cas d'erreur.
	 */
	public static function getList(&$makelist, Closure $get_url,
	int $category_id, string $perms = '1=1', string $password = '1 AS password_auth')
	{
		// Récupération des catégories.
		$sql = "SELECT cat.cat_id,
					   cat.user_id,
					   cat.password_id,
					   cat_parents,
					   cat_name,
					   cat_url,
					   cat_filemtime,
					   cat_uploadable,
					   cat_status,
					   p.cat_id AS p_cat_id,
					   %s,
					   CASE WHEN cat.cat_id > 1
							THEN parent_id
							ELSE 0
							 END AS parent_id,
					   CASE WHEN cat_filemtime IS NULL
							THEN 'category'
							ELSE 'album'
							 END AS cat_type
				  FROM {categories} AS cat
			 LEFT JOIN {passwords} AS p USING (password_id)
				 WHERE %s
			  ORDER BY LOWER(cat_name) ASC";
		if (!DB::execute(sprintf($sql, $password, $perms)))
		{
			return FALSE;
		}
		$categories = DB::fetchAll('cat_id');

		// Construction de la liste.
		$makelist = function(array $categories, int $id = 0, int $level = 0)
		use (&$makelist, $get_url, $category_id): array
		{
			$sublist = [];
			foreach ($categories as &$i)
			{
				if ($i['parent_id'] != $id)
				{
					continue;
				}

				// Ajout de la catégorie à la liste.
				$sublist[] = ['node' => 'element_start'];
				$sublist[] =
				[
					'node' => 'content',
					'current' => (int) ($category_id && $i['cat_id'] == $category_id),
					'level' => $level,
					'id' => $i['cat_id'],
					'title' => $i['cat_id'] > 1 ? $i['cat_name'] : __('galerie'),
					'type' => $i['cat_type'],

					// Lien vers la catégorie.
					'link' => $get_url($i['cat_type'] . '/' . $i['cat_id']),

					// Lien vers la catégorie avec nom d'URL.
					'link_urlname' => $get_url($i['cat_type'] . '/' . $i['cat_id'] . '-'
						. ($i['cat_id'] > 1 ? $i['cat_url'] : __('galerie'))),

					// Lien vers la page de connexion à la catégorie.
					'link_password' => $get_url(
						'password/' . $i['cat_type'] . '/' . $i['cat_id']
					),

					// Indique si un mot de passe a été défini sur la catégorie.
					'password' => $i['p_cat_id'] == $i['cat_id'],

					// Indique si l'utilisateur n'a pas l'autorisation d'accèder
					// à une catégorie protégée par mot de passe.
					'locked' => (bool) !$i['password_auth'],

					// Indique si l'utilisateur a l'autorisation d'accèder
					// à une catégorie protégée par mot de passe.
					'unlocked' => $i['password_id'] && $i['password_auth']
				];
				if ($i['cat_type'] == 'category')
				{
					if ($r = $makelist($categories, (int) $i['cat_id'], $level + 1))
					{
						$sublist = array_merge($sublist, $r);
					}
				}
				$sublist[] = ['node' => 'element_end'];

				// Si c'est un album, on le supprime du tableau des catégories.
				if ($i['cat_type'] == 'album')
				{
					unset($categories[$i['cat_id']]);
				}
			}
			$list = [];
			if ($sublist)
			{
				$list[] = ['node' => 'list_start'];
				$list = array_merge($list, $sublist);
				$list[] = ['node' => 'list_end'];
				
			}
			return $list;
		};

		return $categories;
	}

	/**
	 * Retourne la liste des critères de tri des objets d'une catégorie.
	 *
	 * @return array
	 */
	public static function getOrderByParams(): array
	{
		static $order, $category, $album;

		if (!$order)
		{
			$order =
			[
				'ASC' => __('Croissant'),
				'DESC' => __('Décroissant')
			];
			$category =
			[
				'default' => '*' . __('Par défaut'),
				'cat_position' => __('Tri manuel'),
				'cat_name' => __('Titre (alphabétique)'),
				'cat_name_num' => __('Titre (numérique)'),
				'cat_path' => __('Nom de répertoire (alphabétique)'),
				'cat_path_num' => __('Nom de répertoire (numérique)'),
				'cat_crtdt' => __('Date de création'),
				'cat_lastadddt' => __('Date de mise à jour'),
				'cat_a_size' => __('Poids'),
				'cat_a_items' => __('Nombre de fichiers')
			];
			$album =
			[
				'default' => '*' . __('Par défaut'),
				'item_position' => __('Tri manuel'),
				'item_name' => __('Titre (alphabétique)'),
				'item_name_num' => __('Titre (numérique)'),
				'item_path' => __('Nom de fichier (alphabétique)'),
				'item_path_num' => __('Nom de fichier (numérique)'),
				'item_size' => __('Dimensions'),
				'item_filesize' => __('Poids'),
				'item_hits' => __('Nombre de vues'),
				'item_comments' => __('Nombre de commentaires'),
				'item_votes' => __('Nombre de votes'),
				'item_rate' => __('Note moyenne'),
				'item_favorites' => __('Nombre de favoris'),
				'item_adddt' => __('Date d\'ajout'),
				'item_crtdt' => __('Date de création')
			];
		}

		return ['order' => $order, 'category' => $category, 'album' => $album];
	}

	/**
	 * Retourne l'identifiant des catégories parentes
	 * d'une catégorie sous forme de tableau.
	 *
	 * @param string $parents_id
	 *   Identifiants des catégories parentes dans le même
	 *   format que celui utilisé en base de données avec
	 *   la colonne 'cat_parents' de la table 'categories'.
	 *
	 * @return array
	 */
	public static function getParentsIdArray(string $parents_id): array
	{
		return explode(self::PARENTS_SEPARATOR, substr($parents_id, 0, -1));
	}

	/**
	 * Retourne le chemin d'une catégorie dans laquelle
	 * un utilisateur va ajouter des fichiers.
	 * Si l'utilisateur n'a pas les permissions pour accéder
	 * à la catégorie, alors la chaîne retournée sera vide.
	 *
	 * @param int $cat_id
	 *   Identifiant de la catégorie.
	 * @param bool $from_admin
	 *   L'ajout de fichiers se fait-il depuis l'interface d'administration ?
	 *
	 * @return string|bool
	 *   Retourne le chemin de la catégorie,
	 *   une chaîne vide si l'utilisateur n'a pas les permissions d'y accèder
	 *   ou FALSE en cas d'erreur.
	 */
	public static function getPathUpload(int $cat_id, bool $from_admin = FALSE)
	{
		$from_admin = Auth::$isAdmin ? $from_admin : FALSE;
		$sql = "";
		$params = [$cat_id];

		// Depuis l'administration.
		if ($from_admin)
		{
			$sql = "SELECT cat_path
					  FROM {categories}
					 WHERE cat_id = ?
					   AND cat_filemtime IS NOT NULL";
		}

		// Depuis la galerie.
		else
		{
			// L'utilisateur doit-il être propriétaire de la catégorie ?
			if (Auth::$groupPerms['upload_owner'])
			{
				$sql = " AND user_id = ?";
				$params[] = Auth::$id;
			}

			// Récupération du chemin de l'album.
			$sql = "SELECT *
					  FROM {categories} AS cat
					 WHERE %s
					   AND %s
					   AND cat_id = ?
					   AND cat_status = '1'
					   AND cat_filemtime IS NOT NULL
						   $sql";
			$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		}
		if (!DB::execute($sql, $params))
		{
			return FALSE;
		}
		if (!$cat_infos = DB::fetchRow())
		{
			return '';
		}

		// L'ajout de fichiers est-il autorisé pour cet album ?
		if (!$from_admin)
		{
			$cat_infos = [$cat_infos];
			Parents::settings($cat_infos);
			$cat_infos = $cat_infos[0];
			if (!$cat_infos['parent_uploadable'] || !$cat_infos['cat_uploadable'])
			{
				return '';
			}
		}

		return $cat_infos['cat_path'];
	}

	/**
	 * Déplace des catégories.
	 *
	 * @param array $categories_id
	 *   Identifiant des catégories.
	 * @param int $dest_id
	 *   Identifiant de la catégorie destination.
	 *
	 * @return int
	 *   Retourne le nombre de catégories affectées
	 *   ou -1 en cas d'erreur.
	 */
	public static function move(array $categories_id, int $dest_id): int
	{
		if (!$categories_id)
		{
			return 0;
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return -1;
		}

		// Récupération des informations de la catégorie destination.
		$sql = "SELECT password_id,
					   cat_path,
					   cat_parents
				  FROM {categories}
				 WHERE cat_id = ?
				   AND cat_filemtime IS NULL";
		if (!DB::execute($sql, $dest_id))
		{
			goto dbfail;
		}
		if (!$dest_infos = DB::fetchRow())
		{
			goto nochange;
		}
		$dest_path = $dest_infos['cat_path'];
		$dest_path = ($dest_path == '.') ? '' : $dest_path . '/';
		$dest_parents_id = self::getParentsIdArray($dest_infos['cat_parents']);
		if ($dest_id > 1)
		{
			$dest_parents_id[] = $dest_id;
		}
		$cat_parents = implode(self::PARENTS_SEPARATOR, $dest_parents_id);
		$cat_parents .= self::PARENTS_SEPARATOR;
		$dest_password = $dest_infos['password_id'];

		// Récupération des informations des catégories.
		if (!($categories_by_parent = self::_getCategoriesByParent($categories_id)))
		{
			goto nochange;
		}

		// On traite les catégories catégorie parente par catégorie parente.
		$params = [];
		$update_parents_id = [];
		foreach ($categories_by_parent as &$categories_infos)
		{
			$stats =
			[
				'albums' => 0,
				'comments' => 0,
				'favorites' => 0,
				'hits' => 0,
				'images' => 0,
				'rating' => 0,
				'size' => 0,
				'videos' => 0,
				'votes' => 0
			];
			$cat_update = ['a' => $stats, 'd' => $stats];
			$categories_moved = 0;
			foreach ($categories_infos as &$i)
			{
				// On vérifie que la destination n'est pas identique
				// à la catégorie parente ou qu'elle n'est pas un descendant
				// de l'une des catégories.
				if (preg_match('`^' . $i['cat_path'] . '/.*`', $dest_path)
				 || $i['parent_id'] == $dest_id)
				{
					continue;
				}
				$categories_moved++;

				// On transmet le mot de passe de la catégorie destination
				// à la catégorie à déplacer, sauf si cette dernière possède
				// un mot de passe propre, c'est à dire qui a été placé sur
				// cette catégorie et non sur une catégorie parente.
				$password_id = $i['cat_id'] == $i['p_cat_id']
					? $i['password_id']
					: $dest_password;

				// On renomme le répertoire de la catégorie si une catégorie
				// de même nom existe déjà dans la catégorie cible.
				$new_path = $dest_path . basename($i['cat_path']);
				$new_path = File::getSecureFilename($new_path, CONF_ALBUMS_PATH . '/');
				$rename[] = [$i['cat_path'], $new_path];

				if (!self::_updateChilds($i, $new_path, $dest_parents_id, $password_id))
				{
					goto dbfail;
				}

				// On calcule les valeurs des colonnes
				// des catégories parentes à mettre à jour.
				foreach (['a', 'd'] as $s)
				{
					// Nombre d'albums.
					$cat_update[$s]['albums'] += ($i['cat_filemtime'] !== NULL)
						? (($i['cat_status']) ? ($s == 'a' ? '1' : '0') : ($s == 'a' ? '0' : '1'))
						: $i['cat_' . $s . '_albums'];

					// Autres stats.
					self::_parentsUpdateStats($cat_update[$s], $i, $s);
				}

				$params[] =
				[
					'parent_id' => $dest_id,
					'cat_id' => $i['cat_id'],
					'cat_parents' => $cat_parents
				];
			}
			if ($categories_moved < 1)
			{
				continue;
			}

			// Mise à jour des informations des catégories parentes.
			$source_parents_id = self::getParentsIdArray(
				current($categories_infos)['cat_parents']
			);
			$update_parents_id += array_flip($source_parents_id);
			if (!Parents::updateStats($cat_update, '-', '-', $source_parents_id)
			 || !Parents::updateStats($cat_update, '+', '+', $dest_parents_id)
			 || !Parents::updateInfos($source_parents_id)
			 || !Parents::updateInfos($dest_parents_id))
			{
				goto dbfail;
			}
		}

		if (count($params) < 1)
		{
			goto nochange;
		}

		// On change les informations sur les catégories parentes.
		$sql = "UPDATE {categories}
				   SET parent_id = :parent_id,
					   cat_parents = :cat_parents
				 WHERE cat_id = :cat_id";
		if (!DB::execute($sql, $params))
		{
			goto dbfail;
		}
		$categories_moved = DB::rowCount();

		// On met à jour le nombre de sous-catégories.
		if (!Parents::updateSubCats(array_keys($update_parents_id))
		 || !Parents::updateSubCats($dest_parents_id))
		{
			goto dbfail;
		}

		if (CONF_DEV_MODE && Maintenance::dbStats() !== 0)
		{
			trigger_error('Gallery stats error.', E_USER_WARNING);
			goto dbfail;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			goto dbfail;
		}

		// On déplace les catégories.
		foreach ($rename as &$f)
		{
			$old_path = CONF_ALBUMS_PATH . '/' . $f[0];
			$new_path = CONF_ALBUMS_PATH . '/' . $f[1];
			if (!file_exists($old_path))
			{
				continue;
			}
			if (is_dir(dirname($new_path)))
			{
				File::rename($old_path, $new_path);
			}
		}

		return $categories_moved;

		dbfail:
		DB::rollbackTransaction();
		return -1;

		nochange:
		DB::rollbackTransaction();
		return 0;
	}

	/**
	 * Change le propriétaire de plusieurs catégories.
	 *
	 * @param array $categories_id
	 *   Identifiant des catégories.
	 * @param int $user_id
	 *   Identifiant du nouveau propriétaire.
	 *
	 * @return int
	 *   Retourne le nombre de catégories affectées
	 *   ou -1 en cas d'erreur.
	 */
	public static function owner(array $categories_id, int $user_id): int
	{
		// Le propriétaire ne peut pas être l'utilisateur 2 (invité).
		if ($user_id == 2)
		{
			return 0;
		}

		// On vérifie que l'utilisateur existe.
		$sql = "SELECT COUNT(*) FROM {users} WHERE user_id = ?";
		if (!DB::execute($sql, $user_id))
		{
			return -1;
		}
		if (DB::fetchVal() < 1)
		{
			return 0;
		}

		// On change le propriétaire des catégories.
		foreach ($categories_id as &$id)
		{
			$params[] =
			[
				(int) $id, (int) $id,
				'%' . self::PARENTS_SEPARATOR . ((int) $id) . self::PARENTS_SEPARATOR . '%'
			];
		}
		$sql = "UPDATE {categories}
				   SET user_id = $user_id
				 WHERE (cat_id = ? OR parent_id = ? OR cat_parents LIKE ?)
				   AND user_id != $user_id";
		if (!DB::execute($sql, $params))
		{
			return -1;
		}

		return DB::rowCount();
	}

	/**
	 * Réduit la liste des catégories $list pour ne conserver que
	 * les catégories de $categories ainsi que leurs catégories parentes.
	 *
	 * @param string $type
	 *   Type de catégorie à conserver : 'album' ou 'category'.
	 * @param array $list
	 *   Liste des catégories.
	 * @param array $categories
	 *   Identifiants des catégories à conserver.
	 *
	 * @return void
	 */
	public static function reduceList(string $type, array &$list, array &$categories): void
	{
		$list_ids = [1];

		foreach ($list as $id => &$i)
		{
			if (($type == 'album' && $i['cat_filemtime'] !== NULL)
			 || ($type == 'category' && $i['cat_filemtime'] === NULL))
			{
				if (in_array($id, $categories))
				{
					foreach (explode(self::PARENTS_SEPARATOR, $i['cat_parents']) as $pid)
					{
						if (!in_array($pid, $list_ids))
						{
							$list_ids[] = $pid;
						}
					}
				}
				else
				{
					unset($list[$id]);
				}
			}
		}

		foreach ($list as $id => &$i)
		{
			if ((($type == 'album' && $i['cat_filemtime'] === NULL) || $type == 'category')
			&& !in_array($id, $list_ids))
			{
				unset($list[$id]);
			}
		}
	}

	/**
	 * Remet à zéro le nombre de vues de plusieurs catégories.
	 *
	 * @param array $categories_id
	 *   Identifiant des catégories.
	 *
	 * @return int
	 *   Retourne le nombre de catégories affectées
	 *   ou -1 en cas d'erreur.
	 */
	public static function resetHits(array $categories_id): int
	{
		if (!$categories_id)
		{
			return 0;
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return -1;
		}

		// Récupération des informations des catégories.
		if (!($categories_by_parent = self::_getCategoriesByParent($categories_id)))
		{
			goto nochange;
		}

		// On traite les catégories catégorie parente par catégorie parente.
		$nb_categories = 0;
		$params_childs = [];
		$params_items = [];
		$update_cats = [];
		foreach ($categories_by_parent as &$categories_infos)
		{
			$cat_a_hits = 0;
			$cat_d_hits = 0;
			foreach ($categories_infos as &$i)
			{
				// Si la catégorie ne contient aucun fichier ou aucune vue,
				// inutile de faire la mettre à jour.
				if (($i['cat_a_images'] + $i['cat_d_images']
				   + $i['cat_a_videos'] + $i['cat_d_videos']) == 0
				|| ($i['cat_a_hits'] + $i['cat_d_hits']) == 0)
				{
					continue;
				}

				$path_childs = DB::likeEscape($i['cat_path']) . '/%';

				// Nombre de vues.
				$cat_a_hits += $i['cat_a_hits'];
				$cat_d_hits += $i['cat_d_hits'];

				// Mise à jour des fichiers.
				$params_items[] = $path_childs;

				// Mise à jour de la catégorie.
				$update_cats[] = $i['cat_id'];
				$nb_categories++;

				// Mise à jour des catégories enfants, si ce n'est pas un album.
				if ($i['cat_filemtime'] === NULL)
				{
					$params_childs[] = $path_childs;
				}
			}

			// Mise à jour des catégories parentes.
			$parents_ids = self::getParentsIdArray(current($categories_infos)['cat_parents']);
			$sql = "UPDATE {categories}
					   SET cat_a_hits = cat_a_hits - $cat_a_hits,
						   cat_d_hits = cat_d_hits - $cat_d_hits
					 WHERE cat_id IN (" . DB::inInt($parents_ids) . ")";
			if (!DB::execute($sql))
			{
				goto dbfail;
			}
		}

		// Mise à jour des catégories.
		if ($update_cats)
		{
			$sql = "UPDATE {categories}
			           SET cat_a_hits = 0,
					       cat_d_hits = 0
					 WHERE cat_id IN (" . DB::inInt($update_cats) . ")";
			if (!DB::execute($sql))
			{
				goto dbfail;
			}
		}

		// Mise à jour des sous-catégories.
		if ($params_childs)
		{
			$sql_where = str_repeat("cat_path LIKE ? OR ", count($params_items));
			$sql_where = substr($sql_where, 0, -4);
			$sql = "UPDATE {categories}
					   SET cat_a_hits = 0,
						   cat_d_hits = 0
					 WHERE $sql_where";
			if (!DB::execute($sql, $params_childs))
			{
				goto dbfail;
			}
		}

		// Mise à jour des fichiers.
		if ($params_items)
		{
			$sql_where = str_repeat("item_path LIKE ? OR ", count($params_items));
			$sql_where = substr($sql_where, 0, -4);
			$sql = "UPDATE {items}
					   SET item_hits = 0
					 WHERE $sql_where";
			if (!DB::execute($sql, $params_items))
			{
				goto dbfail;
			}
		}

		if (CONF_DEV_MODE && Maintenance::dbStats() !== 0)
		{
			trigger_error('Gallery stats error.', E_USER_WARNING);
			goto dbfail;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			goto dbfail;
		}

		return $nb_categories;

		dbfail:
		DB::rollbackTransaction();
		return -1;

		nochange:
		DB::rollbackTransaction();
		return 0;
	}

	/**
	 * Change l'état (activé, hors ligne) de plusieurs catégories.
	 *
	 * @param array $categories_id
	 *   Identifiant des catégories.
	 * @param int $status
	 *   État (0 ou 1).
	 *
	 * @return int
	 *   Retourne le nombre de catégories affectées
	 *   ou -1 en cas d'erreur.
	 */
	public static function status(array $categories_id, int $status): int
	{
		if (!$categories_id || $status < 0 || $status > 1)
		{
			return 0;
		}

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return -1;
		}

		// Récupération des informations des catégories.
		if (!($categories_by_parent = self::_getCategoriesByParent($categories_id)))
		{
			goto nochange;
		}

		// On traite les catégories par catégorie parente.
		$update_parents_id = [];
		$nb_categories = 0;
		$s = ($status) ? 'd' : 'a';
		foreach ($categories_by_parent as &$categories_infos)
		{
			// On calcule les valeurs des colonnes
			// des catégories parentes à mettre à jour.
			$sql_items = '';
			$sql_categories = '';
			$params = [];
			$cat_update =
			[
				'albums' => 0,
				'comments' => 0,
				'favorites' => 0,
				'hits' => 0,
				'images' => 0,
				'rating' => 0,
				'size' => 0,
				'videos' => 0,
				'votes' => 0
			];
			foreach ($categories_infos as &$i)
			{
				// Si l'état de la catégorie est le même, on l'ignore.
				if ($i['cat_status'] == $status)
				{
					continue;
				}

				$nb_categories += 1;

				// Chemins des enfants à mettre à jour.
				$sql_items .= "item_path LIKE ? OR ";
				$sql_categories .= "cat_id = " . (int) $i['cat_id'] . " OR cat_path LIKE ? OR ";

				// Paramètres des requêtes préparées.
				$params[] = DB::likeEscape($i['cat_path']) . '/%';

				// Nombre d'albums.
				if ($i['cat_filemtime'] === NULL)
				{
					$cat_update['albums'] += $i['cat_' . $s . '_albums'];
				}
				else
				{
					$cat_update['albums'] += 1;
				}

				// Autres stats.
				self::_parentsUpdateStats($cat_update, $i, $s);
			}

			if (!$params)
			{
				continue;
			}

			// On met à jour les fichiers.
			$sql_items = substr($sql_items, 0, -4);
			$sql = "UPDATE {items} SET item_status = '$status' WHERE $sql_items";
			if (!DB::execute($sql, $params))
			{
				goto dbfail;
			}

			// On récupère l'identifiant des catégories enfants.
			$sql_categories = substr($sql_categories, 0, -4);
			$sql = "SELECT cat_id,
						   parent_id,
						   cat_path,
						   cat_filemtime,
						   cat_d_images + cat_d_videos AS cat_d_items
					  FROM {categories}
					 WHERE cat_a_images + cat_d_images
					     + cat_a_videos + cat_d_videos > 0
					   AND ($sql_categories)";
			if (!DB::execute($sql, $params))
			{
				goto dbfail;
			}
			$childs_categories = [];
			foreach (DB::fetchAll('cat_id') as &$i)
			{
				$childs_categories[] = $i;
			}

			// On met à jour la colonne cat_lastadddt des catégories et albums
			// enfants, mais uniquement quand on désactive la catégorie, et
			// seulement pour les sous-catégories contenant au moins un fichier
			// désactivé.
			if ($status == 0)
			{
				$childs_params = [];
				for ($n = 0; $n < count($childs_categories); $n++)
				{
					if ($childs_categories[$n]['cat_d_items'] == 0)
					{
						continue;
					}
					$sql = "SELECT item_adddt
							  FROM {items}
							 WHERE item_path LIKE ?
						  ORDER BY item_status DESC,
								   item_adddt DESC
							 LIMIT 1";
					$item_path = DB::likeEscape($childs_categories[$n]['cat_path']) . '/%';
					if (!DB::execute($sql, $item_path))
					{
						goto dbfail;
					}
					$item_adddt = DB::fetchVal();
					$childs_params[] =
					[
						($item_adddt) ? $item_adddt : NULL,
						$childs_categories[$n]['cat_path']
					];
				}
				if (count($childs_params))
				{
					$sql = "UPDATE {categories} SET cat_lastadddt = ? WHERE cat_path = ?";
					if (!DB::execute($sql, $childs_params))
					{
						goto dbfail;
					}
				}
			}

			// On met à jour les statistiques de toutes les sous-catégories.
			$s1 = ($status) ? 'd' : 'a';
			$s2 = ($status) ? 'a' : 'd';
			$sql = "UPDATE {categories}
					   SET cat_{$s2}_size = cat_{$s2}_size + cat_{$s1}_size,
						   cat_{$s2}_albums = cat_{$s2}_albums + cat_{$s1}_albums,
						   cat_{$s2}_images = cat_{$s2}_images + cat_{$s1}_images,
						   cat_{$s2}_videos = cat_{$s2}_videos + cat_{$s1}_videos,
						   cat_{$s2}_hits = cat_{$s2}_hits + cat_{$s1}_hits,
						   cat_{$s2}_comments = cat_{$s2}_comments + cat_{$s1}_comments,
						   cat_{$s2}_favorites = cat_{$s2}_favorites + cat_{$s1}_favorites,
						   cat_{$s2}_rating = CASE
								WHEN cat_{$s2}_votes + cat_{$s1}_votes > 0
								THEN ((cat_{$s2}_rating * cat_{$s2}_votes)
									+ (cat_{$s1}_rating * cat_{$s1}_votes))
									/ (cat_{$s2}_votes + cat_{$s1}_votes)
								ELSE 0
								 END,
						   cat_{$s2}_votes = cat_{$s2}_votes + cat_{$s1}_votes,
						   cat_{$s2}_subalbs = cat_{$s2}_subalbs + cat_{$s1}_subalbs,
						   cat_{$s2}_subcats = cat_{$s2}_subcats + cat_{$s1}_subcats,
						   cat_{$s1}_size = 0,
						   cat_{$s1}_albums = 0,
						   cat_{$s1}_images = 0,
						   cat_{$s1}_videos = 0,
						   cat_{$s1}_hits = 0,
						   cat_{$s1}_comments = 0,
						   cat_{$s1}_favorites = 0,
						   cat_{$s1}_rating = 0,
						   cat_{$s1}_votes = 0,
						   cat_{$s1}_subalbs = 0,
						   cat_{$s1}_subcats = 0,
						   cat_status = '$status'
					 WHERE $sql_categories";
			if (!DB::execute($sql, $params))
			{
				goto dbfail;
			}

			// Mise à jour des statistiques des catégories parentes.
			$cat_update = ['a' => $cat_update, 'd' => $cat_update];
			if ($status == 1)
			{
				$a_sign = '+';
				$d_sign = '-';
			}
			else
			{
				$a_sign = '-';
				$d_sign = '+';
			}
			$parents_id = self::getParentsIdArray(current($categories_infos)['cat_parents']);
			$update_parents_id += array_flip($parents_id);
			if (!Parents::updateStats($cat_update, $a_sign, $d_sign, $parents_id))
			{
				goto dbfail;
			}

			// On met à jour la date d'ajout du dernier fichier
			// et la vignette des catégories parentes.
			if (!Parents::updateInfos($parents_id))
			{
				goto dbfail;
			}
		}

		if ($nb_categories < 1)
		{
			goto nochange;
		}

		// On met à jour le nombre de sous-catégories.
		if (!Parents::updateSubCats(array_keys($update_parents_id)))
		{
			goto dbfail;
		}

		if (CONF_DEV_MODE && Maintenance::dbStats() !== 0)
		{
			trigger_error('Gallery stats error.', E_USER_WARNING);
			goto dbfail;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			goto dbfail;
		}

		return $nb_categories;

		dbfail:
		DB::rollbackTransaction();
		return -1;

		nochange:
		DB::rollbackTransaction();
		return 0;
	}



	/**
	 * Modifie le nom de répertoire de toutes les sous-catégorie d'une catégorie
	 * et retourne le nouveau nom de répertoire de la catégorie.
	 *
	 * @param array $i
	 *   Informations de la catégorie.
	 * @param string $new_dirname
	 *   Nouveau nom de répertoire.
	 *
	 * @return mixed
	 *   Retourne les informations du nouveau nom de répertoire dans un tableau,
	 *   ou FALSE si une erreur est survenue.
	 */
	private static function _changeDirname(array &$i, string $new_dirname)
	{
		// Nouveau nom de répertoire.
		$new_dirname = substr($new_dirname, 0, 220);
		$parent_path = dirname($i['cat_path']);
		$parents = $parent_path == '.' ? '' : $parent_path . '/';
		$parent_abs_path =  CONF_ALBUMS_PATH . '/' . $parents;

		$new_dirname = App::getValidDirname($new_dirname);
		if ($new_dirname === '' || $new_dirname == basename($i['cat_path']))
		{
			return [$i['cat_path']];
		}

		$new_dirname = File::getSecureFilename($new_dirname, $parent_abs_path);
		$new_path = $parents . $new_dirname;
		if ($new_path == $i['cat_path'])
		{
			return [$i['cat_path']];
		}

		// Modifications des informations des descendants.
		if (!self::_updateChilds($i, $new_path, [], $i['password_id']))
		{
			return FALSE;
		}

		return
		[
			$new_path,
			$parent_abs_path . basename($i['cat_path']),
			$parent_abs_path . basename($new_path)
		];
	}

	/**
	 * Modification du mot de passe.
	 *
	 * @param array $i
	 *   Informations de la catégorie.
	 * @param string $new_password
	 *   Nouveau mot de passe.
	 *
	 * @return mixed
	 *   Informations du mot de passe (array)
	 *   ou FALSE en cas d'erreur.
	 */

	private static function _changePassword(array &$i, string $new_password)
	{
		// Si aucun mot de passe n'est défini, inutile d'aller plus loin.
		if (preg_match('`^\*+$`', $new_password)
		|| ($new_password === '' && $i['password_id'] === NULL))
		{
			return ['change' => FALSE, 'id' => $i['password_id']];
		}

		$old_password_id = $password_id = $i['password_id'];
		$update_childs = FALSE;

		if ($i['parent_id'] > 1)
		{
			// Récupération du mot de passe de la catégorie parente.
			$sql = "SELECT p.*
					  FROM {passwords} AS p
				 LEFT JOIN {categories} AS cat USING (password_id)
					 WHERE cat.cat_id = ?";
			if (!DB::execute($sql, $i['parent_id']))
			{
				return FALSE;
			}
			$parent_password = DB::fetchRow();

			// La catégorie parente possède-t-elle déjà un mot de passe ?
			if ($parent_password)
			{
				// Si oui, on reprend le mot de passe de la catégorie parente
				// si aucun mot de passe n'a été spécifié, ou
				// si le mot de passe de la catégorie parente est
				// le même que celui de la catégorie courante.
				if ($new_password === ''
				|| App::passwordVerify($new_password, $parent_password['password_hash']))
				{
					$password_id = $parent_password['password_id'];
					$update_childs = TRUE;
				}

				// Si les deux mots de passe sont différents,
				// on en crée un nouveau pour la catégorie courante.
				else if ($parent_password['password_id'] == $password_id)
				{
					$password_id = NULL;
					$update_childs = TRUE;
				}
			}
		}

		// Suppression du mot de passe.
		if ($new_password === '' && !$update_childs)
		{
			$sql = "DELETE FROM {passwords} WHERE password_id = ?";
			if (!DB::execute($sql, $password_id))
			{
				return FALSE;
			}
			$password_id = NULL;
		}

		// Modification du mot de passe.
		else if ($password_id)
		{
			$sql = "UPDATE {passwords} SET password_hash = ? WHERE password_id  = ?";
			$params = [App::passwordHash($new_password), $password_id];
			if (!DB::execute($sql, $params))
			{
				return FALSE;
			}
		}

		// Ajout du nouveau mot de passe.
		else
		{
			$sql = "INSERT INTO {passwords} (cat_id, password_hash) VALUES (?, ?)";
			$params = [$i['cat_id'], App::passwordHash($new_password)];
			$seq = ['table' => '{passwords}', 'column' => 'password_id'];
			if (!DB::execute($sql, $params, $seq))
			{
				return FALSE;
			}
			$password_id = DB::lastInsertId();
			$update_childs = TRUE;
		}

		// On met à jour la catégorie courante
		// et toutes ses sous-catégories.
		if ($update_childs)
		{
			$params =
			[
				$password_id,
				$i['cat_parents'] . $i['cat_id'] . '.%',
				$i['cat_id'],
				$old_password_id
			];
			$sql = "UPDATE {categories}
					   SET password_id = ?
					 WHERE (cat_parents LIKE ? OR cat_id = ?)
					   AND (password_id IS NULL OR password_id = ?)";
			if (!DB::execute($sql, $params))
			{
				return FALSE;
			}
		}

		// On supprime les mots de passe de la table {passwords}
		// qui ne sont plus associés à une catégorie.
		$sql = "SELECT DISTINCT password_id FROM {categories} WHERE password_id IS NOT NULL";
		$sql = "DELETE FROM {passwords} WHERE password_id NOT IN ($sql)";
		if (!DB::execute($sql))
		{
			return FALSE;
		}

		// On supprime les associations session - catégorie, afin de
		// déconnecter les utilisateurs qui auraient déverrouiller la
		// catégorie si celle-ci était déjà protégée par un mot de passe.
		$sql = "DELETE FROM {sessions_categories} WHERE cat_id = ?";
		if (!DB::execute($sql, $i['cat_id']))
		{
			return FALSE;
		}

		return ['change' => TRUE, 'id' => $password_id];
	}

	/**
	 * Retourne les informations d'une liste de catégories
	 * groupées par leur catégorie parente.
	 *
	 * @param array $categories_id
	 *   Identifiants des catégories.
	 *
	 * @return array
	 */
	private static function _getCategoriesByParent(array $categories_id): array
	{
		$sql = "SELECT cat.*,
		               p.cat_id AS p_cat_id
				  FROM {categories} AS cat
			 LEFT JOIN {passwords} AS p USING (password_id)
				 WHERE cat.cat_id IN (" . DB::inInt($categories_id) . ")
				   AND cat.cat_id > 1";
		if (!DB::execute($sql))
		{
			return [];
		}
		$categories_infos = DB::fetchAll('cat_id');

		// On réorganise les catégories par catégorie parente.
		$categories_by_parent = [];
		foreach ($categories_infos as $id => &$i)
		{
			// Si une catégorie de $categories_id est une sous-catégorie
			// d'une autre catégorie de $categories_id, on l'ignore.
			foreach (self::getParentsIdArray($i['cat_parents']) as &$parent_id)
			{
				if (isset($categories_infos[$parent_id]))
				{
					continue 2;
				}
			}

			$categories_by_parent[$i['parent_id']][$id] = $i;
		}

		return $categories_by_parent;
	}

	/**
	 * Recalcule les statistiques des catégories parentes.
	 *
	 * @param array $update
	 *   Informations des catégories parentes à mettre à jour.
	 * @param array $i
	 *   Informations de la catégorie.
	 * @param string $s
	 *
	 * @return void
	 */
	private static function _parentsUpdateStats(array &$update, array &$i, string $s): void
	{
		// Note moyenne.
		if ($i['cat_' . $s . '_votes'] > 0)
		{
			$update['rating'] =
				(($update['rating'] * $update['votes'])
				+ ($i['cat_' . $s . '_rating'] * $i['cat_' . $s . '_votes']))
				/ ($update['votes'] + $i['cat_' . $s . '_votes']);
		}

		// Autres stats.
		$update['comments'] += $i['cat_' . $s . '_comments'];
		$update['favorites'] += $i['cat_' . $s . '_favorites'];
		$update['hits'] += $i['cat_' . $s . '_hits'];
		$update['images'] += $i['cat_' . $s . '_images'];
		$update['size'] += $i['cat_' . $s . '_size'];
		$update['videos'] += $i['cat_' . $s . '_videos'];
		$update['votes'] += $i['cat_' . $s . '_votes'];
	}

	/**
	 * Met à jour les informations suivantes pour une catégorie, ainsi
	 * que pour toutes les sous-catégories et fichiers qu'elle contient :
	 *
	 * - Chemin (cat_path et item_path).
	 * - Identifiants des catégories parentes (cat_parents).
	 * - Mot de passe (password_id).
	 *
	 * @param array $i
	 * @param string $new_path
	 * @param array $parents_id
	 * @param mixed $password_id
	 *
	 * @return bool
	 */
	private static function _updateChilds(array &$i, string $new_path,
	array $parents_id, $password_id): bool
	{
		$params =
		[
			'cat_id' => $i['cat_id'],
			'new_path' => $new_path,
			'old_path' => $i['cat_path'],
			'old_path_like' => DB::likeEscape($i['cat_path']) . '/%',
			'password_id' => $password_id
		];

		// Colonne 'cat_parents'.
		$sql_cat_parents = "";
		if ($parents_id)
		{
			$cat_parents = implode(self::PARENTS_SEPARATOR, $parents_id)
				. self::PARENTS_SEPARATOR;
			$cat_parents_childs = $cat_parents . $i['cat_id'] . self::PARENTS_SEPARATOR;
			$params['cat_parents'] = $cat_parents;
			$params['cat_parents_childs'] = $cat_parents_childs;
			$sql_cat_parents = "cat_parents = CASE
				WHEN cat_id = :cat_id
				THEN :cat_parents
				ELSE :cat_parents_childs END,";
		}

		// On modifie les informations de la catégorie
		// et de toutes les catégories descendantes.
		$sql = "UPDATE {categories}
				   SET $sql_cat_parents
				       cat_path = REPLACE('^'||cat_path, '^'||:old_path, :new_path),
				       password_id = :password_id
				 WHERE cat_id = :cat_id
					OR cat_path LIKE :old_path_like";
		if (!DB::execute($sql, $params))
		{
			return FALSE;
		}
		$categories_affected = DB::rowCount();

		// On vérifie que la colonne 'cat_parents'
		// des sous-catégories a bien été modifiée.
		if (CONF_DEV_MODE && $parents_id)
		{
			$sql = "SELECT COUNT(cat_id)
					  FROM {categories}
					 WHERE cat_parents LIKE :cat_parents";
			$params =
			[
				'cat_parents' => $cat_parents_childs . '%',
			];
			if (!DB::execute($sql, $params))
			{
				return FALSE;
			}
			if (($count = DB::fetchVal()) != ($categories_affected - 1))
			{
				trigger_error('Error: ' . __LINE__, E_USER_WARNING);
				return FALSE;
			}
		}

		// On vérifie que le chemin des catégories a bien été changé.
		if (CONF_DEV_MODE)
		{
			$sql = "SELECT COUNT(cat_id)
					  FROM {categories}
					 WHERE cat_id = :cat_id
						OR cat_path LIKE :new_path_like";
			$params =
			[
				'cat_id' => $i['cat_id'],
				'new_path_like' => DB::likeEscape($new_path) . '/%',
			];
			if (!DB::execute($sql, $params))
			{
				return FALSE;
			}
			if (($count = DB::fetchVal()) != $categories_affected)
			{
				trigger_error('Error: ' . __LINE__, E_USER_WARNING);
				return FALSE;
			}
		}

		// On modifie le chemin de tous les fichiers de la catégorie.
		$nb_items = $i['cat_a_images'] + $i['cat_d_images']
				  + $i['cat_a_videos'] + $i['cat_d_videos'];
		$sql = "UPDATE {items}
				   SET item_path = REPLACE('^'||item_path, '^'||:old_path, :new_path)
				 WHERE item_path LIKE :old_path_like";
		$params =
		[
			'new_path' => $new_path,
			'old_path' => $i['cat_path'],
			'old_path_like' => DB::likeEscape($i['cat_path']) . '/%'
		];
		if (!DB::execute($sql, $params))
		{
			return FALSE;
		}

		if (CONF_DEV_MODE)
		{
			$row_count = DB::rowCount();
			if ($row_count != $nb_items)
			{
				trigger_error('Error: ' . __LINE__, E_USER_WARNING);
				return FALSE;
			}
		}

		return TRUE;
	}
}
?>