<?php
declare(strict_types = 1);

/**
 * Gestionnaire de cookies.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Cookie
{
	/**
	 * Nom du cookie.
	 *
	 * @var string
	 */
	private $_name;

	/**
	 * Contenu du cookie.
	 *
	 * @var array
	 */
	private $_value = [];



	/**
	 * Initialisation : récupération du contenu du cookie.
	 *
	 * @param string $name
	 *   Nom du cookie.
	 *
	 * @return void
	 */
	public function __construct(string $name)
	{
		$this->_name = $name;

		if (isset($_COOKIE[$name]) && Utility::isJson($_COOKIE[$name]))
		{
			$this->_value = Utility::jsonDecode($_COOKIE[$name]);
		}
	}

	/**
	 * Ajoute des données au cookie.
	 *
	 * @param string $k
	 * @param string $v
	 *
	 * @return void
	 */
	public function add(string $k, string $v): void
	{
		$this->_value[$k] = $v;
	}

	/**
	 * Supprime tout ou une partie du cookie.
	 *
	 * @param string $k
	 *
	 * @return void
	 */
	public function delete(string $k = ''): void
	{
		if ($k)
		{
			unset($this->_value[$k]);
		}
		else
		{
			$this->_value = [];
		}
	}

	/**
	 * Lecture d'une valeur contenue dans le cookie.
	 *
	 * @param string $k
	 *
	 * @return string
	 */
	public function read(string $k): string
	{
		return array_key_exists($k, $this->_value) ? $this->_value[$k] : '';
	}

	/**
	 * Écriture du cookie.
	 *
	 * @param int $expire
	 *   Durée d'expiration du cookie.
	 *
	 * @return bool
	 */
	public function write(int $expire = 0): bool
	{
		// Encodage des données.
		$value = Utility::jsonEncode($this->_value);

		// On supprime les données par sécurité.
		$this->_value = [];

		// Durée d'expiration.
		// Ne fonctionne pas comme attendu avec certains navigateurs si $expire = 0.
		$expire = $expire ? time() + $expire : 0;

		// Chemin.
		switch (CONF_GALLERY_PATH)
		{
			case '' :
				$path = App::getGalleryPath() . '/';
				break;

			case '/' :
				$path = '/';
				break;

			default :
				$path = CONF_GALLERY_PATH . '/';
		}

		// Domaine.
		$domain = preg_replace('`:\d+$`', '', $_SERVER['HTTP_HOST']);
		$domain = ($domain == 'localhost' || $domain == '127.0.0.1') ? '' : $domain;

		// Écriture du cookie.
		return setcookie($this->_name, $value, $expire, $path, $domain, FALSE, TRUE);
	}
}
?>