<?php
declare(strict_types = 1);

/**
 * Méthodes de gestion du HTML.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class HTML
{
	/**
	 * Liste des attributs de balises autorisés.
	 * Retourne la regexp pour la valeur de l'attribut $attr.
	 *
	 * @param string $attr
	 *
	 * @return mixed
	 */
	public static function allowedAttrs(string $attr = '')
	{
		$allowed_attrs =
		[
			'alt'      => '[-_\w\d\s,;:.%]+',
			'cite'     => '[-_\w\d\s,;:.%]+',
			'class'    => '[-_a-z0-9\s]+',
			'datetime' => '[-0-9]+',
			'height'   => '\d{1,6}',
			'href'     => '[-_\w\d@&=+?%~./,;:*#]+',
			'hreflang' => '[-a-z]{2,5}',
			'id'       => '[_a-z0-9]+',
			'lang'     => '[-a-z]{2,5}',
			'max'      => '[-+0-9e.]+',
			'pubdate'  => '[-0-9]+',
			'rel'      => '[-_a-z0-9]+',
			'src'      => '[-_a-z0-9./:=\?]+',
			'style'    => '[-_a-z0-9\s;:#,%.]+',
			'target'   => '[-_a-z0-9]+',
			'title'    => '[-_\w\d\s@&=+!?%°~./,;:*#()\[\]\|]+',
			'value'    => '[-+0-9e.]+',
			'width'    => '\d{1,6}'
		];

		if ($attr === '')
		{
			return $allowed_attrs;
		}

		return $allowed_attrs[$attr] ?? FALSE;
	}

	/**
	 * Retourne la liste des balises autorisées.
	 *
	 * @return array
	 */
	public static function allowedTags(): array
	{
		// Toutes les balises de formulaires, d'objets (script, ...)
		// et de structure de page (body, ...) ne sont pas autorisées.
		return
		[
			'a', 'abbr', 'address', 'article', 'aside', 'blockquote',
			'caption', 'cite', 'code', 'colgroup', 'del', 'dfn',
			'div', 'dd', 'dl', 'dt', 'em', 'figcaption', 'figure',
			'footer', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'header',
			'hr', 'img', 'ins', 'kbd', 'legend', 'li', 'mark', 'nav',
			'ol', 'p', 'pre', 'progress', 'q', 'rt', 'samp',
			'section', 'small', 'span', 'strong', 'sub', 'sup',
			'table', 'tbody', 'td', 'tfoot', 'th', 'thead', 'time',
			'tr', 'ul', 'var'
		];
	}

	/**
	 * Décode le code HTML autorisé d'une chaîne.
	 *
	 * @param string $str
	 *
	 * @return string
	 */
	public static function decode(string $str): string
	{
		$decode = function(array $str): string
		{
			// "\" forcément plus que suspect.
			if (strstr($str[0], '\\'))
			{
				return $str[0];
			}

			// Suppression des chevrons ouvrant et fermant
			// pour les balises ouvrantes et fermantes.
			$s = preg_replace('`^&lt;/?|&gt;$`', '', $str[0]);

			// Récupèration du nom de la balise.
			$tag = explode(' ', $s, 2);

			// Création d'un tableau contenant les attributs de la balise.
			$s = trim(preg_replace('`^' . $tag[0] . ' `', '', $s));
			$attrs = array_map('trim', preg_split('`(?<=&quot;) `', $s, 15));

			// Est-ce une balise autorisée ?
			$ok = FALSE;
			if (in_array($tag[0], self::allowedTags()))
			{
				$ok = TRUE;

				// Si la balise contient des attributs, on les analyse un à un.
				if (isset($attrs[1]) || $attrs[0] != $tag[0])
				{
					// Suppression du slash de fermeture pour les balises atomiques.
					if (($tag[0] == 'img' || $tag[0] == 'hr')
					&& $attrs[count($attrs) - 1] == '/')
					{
						unset($attrs[count($attrs) - 1]);
					}

					// Vérification de chaque attribut.
					for ($i = 0, $count = count($attrs); $i < $count; $i++)
					{
						$attr = explode('=', $attrs[$i], 2);

						// Est-ce un attribut autorisé ?
						if (self::allowedAttrs($attr[0]) === FALSE)
						{
							$ok = FALSE;
							break;
						}

						// Filtrage de l'attribut "href".
						if ($attr[0] == 'href')
						{
							if (preg_match('`^&quot;\s*javascript:..+&quot;$`i', $attr[1]))
							{
								$ok = FALSE;
								break;
							}
						}

						// Filtrage de l'attribut "src".
						if ($attr[0] == 'src')
						{
							$regex = '(?:' . preg_quote(GALLERY_HOST) . ')?';
							$regex .= preg_quote(CONF_GALLERY_PATH);
							$regex .= '/(?:image\.php\?id=\d{1,11}|';
							$regex .= 'images/[/0-9a-z_-]+\.(?:jpe?g|gif|png))';
							$regex = '`^&quot;' . $regex . '&quot;$`i';

							if (!preg_match($regex, $attr[1]))
							{
								$ok = FALSE;
								break;
							}
						}

						// Vérification de la présence d'entités HTML autres que &#039; et &amp;
						$value = preg_replace('`^' . $attr[0]
							. '=&quot;|&quot;$`', '', $attrs[$i]);
						$test = str_replace(['&#039;', '&amp;'], '', $value);
						if (strstr($test, '&'))
						{
							$ok = FALSE;
							break;
						}

						// Vérification de la valeur de l'attribut.
						$regex = '`^' . self::allowedAttrs($attr[0]) . '$`ui';
						if (!preg_match($regex, $value))
						{
							$ok = FALSE;
							break;
						}
					}
				}
			}

			// Si la balise est OK, on décode les caractères convertis par
			// htmlspecialchars(), sinon on renvoi la chaîne sans la modifier.
			return ($ok)
				? htmlspecialchars_decode($str[0], ENT_QUOTES)
				: $str[0];
		};

		return preg_replace_callback('`&lt;(?:(?!&gt;).){1,255}&gt;`', $decode, $str);
	}

	/**
	 * Convertit les URLs d'une chaîne en liens HTML.
	 *
	 * @param string $str
	 * @param int $limit
	 *   Nombre maximum de caractères du lien.
	 *
	 * @return string
	 */
	public static function linkify(string $str, int $limit = 0): string
	{
		$regexp = '`((?<!&quot;)(?<!")' . Utility::URLRegexp() . ')`i';
		$str = preg_replace($regexp, '<a href="$1">$1</a>', $str);

		return ($limit)
			? preg_replace(
				'`(<a[^>]+>)(.{' . $limit . '}).*(</a>)`ui',
				'\\1\\2...\\3',
				$str)
			: $str;
	}

	/**
	 * Applique la fonction htmlspecialchars à une chaîne
	 * ou aux valeurs d'un tableau ou d'un objet.
	 *
	 * @param mixed $s
	 * @param bool $double_encode
	 *
	 * @return mixed
	 */
	public static function specialchars(&$s, bool $double_encode = TRUE)
	{
		if (is_string($s))
		{
			$s = defined('ENT_DISALLOWED')
				? htmlspecialchars($s, ENT_DISALLOWED | ENT_QUOTES, 'UTF-8', $double_encode)
				: htmlspecialchars($s, ENT_QUOTES, 'UTF-8', $double_encode);
		}
		else if (is_array($s))
		{
			array_walk($s, [__CLASS__, __FUNCTION__]);
		}
		return $s;
	}
}
?>