<?php
declare(strict_types = 1);

/**
 * Mise à jour de l'application.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class Update
{
	/**
	 * Regexp pour le numéro de version.
	 *
	 * @var string
	 */
	const VERSION_REGEXP = '`^\d{1,3}(?:\.\d{1,3}){1,3}(?:\-(?:alpha|beta)-\d{1,3})?$`';



	/**
	 * Peut-on récupérer le contenu d'un fichier distant ?
	 *
	 * @return bool
	 */
	public static function isAllowedHTTPOrigin(): bool
	{
		// file_get_contents
		if (function_exists('ini_get') && System::getIniBool('allow_url_fopen'))
		{
			return TRUE;
		}

		// cURL
		if (function_exists('curl_init'))
		{
			return TRUE;
		}

		return FALSE;
	}

	/**
	 * Récupère et vérifie l'archive de la version $version.
	 *
	 * @param array $version
	 *
	 * @return bool
	 */
	public static function getArchive(array $version): bool
	{
		if (!preg_match(self::VERSION_REGEXP, (string) $version[0])
		|| !preg_match('`^[a-f0-9]{32}$`', (string) $version[1]))
		{
			trigger_error('regexp error', E_USER_WARNING);
			return FALSE;
		}

		$source = System::APP_WEBSITE . 'igalerie-' . $version[0] . '.zip?' . Utility::key(32);
		$dest = GALLERY_ROOT . '/latestversion.zip';
		$data = self::_getContents($source);

		if (!$data || File::putContents($dest, $data) === FALSE)
		{
			trigger_error('file error', E_USER_WARNING);
			return FALSE;
		}

		if (md5_file($dest) != $version[1])
		{
			trigger_error('md5 error', E_USER_WARNING);
			return FALSE;
		}

		if (!filesize($dest))
		{
			trigger_error('filesize error', E_USER_WARNING);
			return FALSE;
		}

		return TRUE;
	}

	/**
	 * Retourne les informations de la dernière version de l'application
	 * sous forme d'un tableau, ou FALSE si une erreur est survenue.
	 *
	 * @return mixed
	 */
	public static function getLatestVersion()
	{
		$url = System::APP_WEBSITE . 'latestversion.txt?' . Utility::key(32);
		if (!$latest_version = self::_getContents($url))
		{
			trigger_error('file error', E_USER_WARNING);
			return FALSE;
		}

		$latest_version = explode(':', $latest_version);
		if (!is_array($latest_version) || count($latest_version) !== 2)
		{
			trigger_error('array error', E_USER_WARNING);
			return FALSE;
		}

		if (preg_match(self::VERSION_REGEXP, (string) $latest_version[0])
		&& preg_match('`^[a-f0-9]{32}$`', (string) $latest_version[1]))
		{
			return $latest_version;
		}

		trigger_error('regexp error', E_USER_WARNING);
		return FALSE;
	}

	/**
	 * Extrait les fichiers de l'archive et remplace
	 * tous les fichiers de l'application.
	 *
	 * @return bool
	 */
	public static function replaceFiles(): bool
	{
		$zip_file = GALLERY_ROOT . '/latestversion.zip';

		$zip = new Zip;
		if ($zip->open($zip_file) !== TRUE)
		{
			trigger_error('zip error', E_USER_WARNING);
			return FALSE;
		}

		$zip->extractSubdirTo(dirname($zip_file), 'igalerie');
		$zip->close();
		File::unlink($zip_file);

		return TRUE;
	}



	/**
	 * Peut-on récupérer le contenu d'un fichier distant ?
	 *
	 * @return mixed
	 */
	private static function _getContents(string $url)
	{
		// file_get_contents
		if (function_exists('ini_get') && System::getIniBool('allow_url_fopen'))
		{
			return file_get_contents($url);
		}

		// cURL
		if (function_exists('curl_init'))
		{
			$ch = curl_init();
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($ch, CURLOPT_URL, $url);
			$data = curl_exec($ch);
			curl_close($ch);

			return $data;
		}

		return FALSE;
	}
}
?>