<?php
declare(strict_types = 1);

/**
 * Traitement des requêtes Ajax pour la galerie.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
class AjaxGallery extends Ajax
{
	/**
	 * Vérification du mot de passe de catégorie.
	 *
	 * @return void
	 */
	public static function checkCatPassword(): void
	{
		require_once(__DIR__ . '/GalleryCategory.class.php');

		$r = GalleryCategory::checkPassword('category', (int) $_POST['cat_id'],
			$_POST['password'] ?? '', !empty($_POST['remember']));

		if ($r == 'db_error')
		{
			self::_dbError();
		}
		if ($r == 'not_found')
		{
			self::_forbidden();
		}
		if ($r == 'bruteforce')
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => 'Brute force attack detected.'
			]);
		}
		if (substr($r, 0, 9) == 'redirect:')
		{
			self::_printResponse(
			[
				'status' => 'success',
				'url' => App::getURL(substr($r, 9))
			]);
		}

		self::_printResponse(
		[
			'status' => 'error',
			'message' => __('Mot de passe incorrect.')
		]);
	}

	/**
	 * Édition des informations d'un fichier :
	 * titre, description et tags.
	 *
	 * @return void
	 */
	public static function editItem(): void
	{
		// Vérifications.
		if (!isset($_POST['item_id']) || !preg_match('`^\d{1,12}$`', $_POST['item_id']))
		{
			self::_forbidden();
		}
		$id = (int) $_POST['item_id'];

		// Permissions d'accès au fichier.
		$item_infos = self::_itemPermission($id);

		// Permissions de modification du fichier.
		if ((Auth::$isAdmin || (Config::$params['users']
		&& $item_infos['user_id'] == Auth::$id)) === FALSE)
		{
			self::_forbidden();
		}

		// Préparation des données.
		$data = [];
		if (isset($_POST['title']))
		{
			$data['item_name'] = $data['item_url'] = $_POST['title'];
		}
		if (isset($_POST['description']))
		{
			$data['item_desc'] = $_POST['description'];
		}
		if (Config::$params['tags'] && isset($_POST['tags']))
		{
			$data['tags'] = $_POST['tags'];
		}
		if (!$data)
		{
			self::_forbidden();
		}

		// Mise à jour de la base de données.
		$data = [$id => $data];
		$r = Item::edit($data);

		// Message de rapport.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
		}
		else if ($r > 0)
		{
			// Log d'activité.
			App::logActivity('item_edit', '', ['item_id' => $_POST['item_id']]);

			$sql = "SELECT item_id,
						   item_name,
						   item_desc
					  FROM {items}
					 WHERE item_id = ?";
			if (!DB::execute($sql, $id))
			{
				self::_dbError();
			}
			$i = DB::fetchRow();

			$params =
			[
				'status' => 'success',
				'message' => Report::getSuccessDefaultMessage()
			];
			if (isset($_POST['title']))
			{
				$params['title'] = HTML::specialchars($i['item_name']);
			}
			if (isset($_POST['description']))
			{
				$params['description'] = $i['item_desc'] === NULL
					? NULL
					: Template::formatDescription(HTML::specialchars($i['item_desc']));
			}
			if (Config::$params['tags'] && isset($_POST['tags']))
			{
				require_once(__DIR__ . '/GalleryItems.class.php');
				$tags = GalleryItems::getTags($i);
				$tags_list = [];
				foreach ($tags as &$i)
				{
					$tags_list[] = $i['name'];
				}
				$params['tags_list'] = $tags_list;
				$params['tags'] = HTML::specialchars($tags);
			}
			self::_printResponse($params);
		}
		else
		{
			self::_printResponse(
			[
				'status' => 'success',
				'message' => Report::getNoChangeDefaultMessage()
			]);
		}
	}

	/**
	 * Supprime un fichier.
	 *
	 * @return void
	 */
	public static function deleteItem(): void
	{
		// Vérifications.
		if (!isset($_POST['item_id']) || !preg_match('`^\d{1,12}$`', $_POST['item_id']))
		{
			self::_forbidden();
		}
		$id = (int) $_POST['item_id'];

		// Permissions d'accès au fichier.
		$item_infos = self::_itemPermission($id);

		// Permissions de suppression du fichier.
		if ((Auth::$isAdmin || (Config::$params['users']
		&& $item_infos['user_id'] == Auth::$id)) === FALSE)
		{
			self::_forbidden();
		}

		$r = Item::delete([$id]);

		// Message de rapport.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
		}
		else if ($r > 0)
		{
			self::_printResponse(
			[
				'status' => 'success',
				'message' => __('Le fichier a été supprimé.'),
				'redirect' => CONF_GALLERY_PATH . '/'
			]);
		}
	}

	/**
	 * Ajoute ou retire un fichier des favoris d'un utilisateur.
	 *
	 * @param string $action
	 *   Action à effectuer : 'add' ou 'remove'.
	 *
	 * @return void
	 */
	public static function favorites(string $action): void
	{
		// Quelques vérifications.
		if (!Config::$params['users'] || !isset($_POST['item_id'])
		|| !preg_match('`^\d{1,12}$`', $_POST['item_id']))
		{
			self::_forbidden();
		}

		// Permissions d'accès au fichier.
		self::_itemPermission((int) $_POST['item_id']);

		// On modifie les favoris.
		$r = Item::favorites([$_POST['item_id']], $action, Auth::$id);
		if ($r == 0)
		{
			self::_forbidden();
		}
		if ($r == -1)
		{
			self::_dbError();
		}

		// Log d'activité.
		App::logActivity('favorites_' . $action, '', ['item_id' => $_POST['item_id']]);

		// Récupération du nombre de favoris du fichier.
		$sql = "SELECT item_favorites FROM {items} WHERE item_id = ?";
		$item_favorites = (int) (DB::execute($sql, $_POST['item_id']) ? DB::fetchVal() : 0);

		self::_printResponse([
			'favorites' => $item_favorites,
			'favorites_short' => L10N::formatShortNumber($item_favorites),
			'status' => 'success'
		]);
	}

	/**
	 * Connexion utilisateur.
	 *
	 * @return void
	 */
	public static function login(): void
	{
		if (Auth::form($_POST['login'], $_POST['password'], !empty($_POST['remember'])))
		{
			self::_printResponse(
			[
				'status' => 'success'
			]);
		}
		else
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => __('Informations incorrectes.')
			]);
		}
	}

	/**
	 * Enregistrement des paramètres de thème.
	 *
	 * @return void
	 */
	public static function themeParams(): void
	{
		// Vérifications.
		if (!Auth::$isAdmin || !isset($_POST['params']) || !is_array($_POST['params'])
		|| !isset($_POST['template']) || !preg_match('`^[a-z0-9_]{1,50}$`i', $_POST['template'])
		|| $_POST['template'] != Config::$params['theme_template'])
		{
			self::_forbidden();
		}

		if (mb_strlen(Utility::jsonEncode($_POST['params'])) > 10000)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => 'Array too large.'
			]);
		}

		// Enregistrement.
		$params = is_array(Config::$params['theme_params'])
			? Config::$params['theme_params']
			: [];
		$params[$_POST['template']] = $_POST['params'];
		$r = Config::changeDBParams(['theme_params' => $params]);

		// Message de rapport.
		if ($r == -1)
		{
			self::_printResponse(
			[
				'status' => 'error',
				'message' => Report::getErrorDefaultMessage()
			]);
		}
		else if ($r > 0)
		{
			self::_printResponse(
			[
				'status' => 'success',
				'message' => Report::getSuccessDefaultMessage()
			]);
		}
		else
		{
			self::_printResponse(
			[
				'status' => 'success',
				'message' => Report::getNoChangeDefaultMessage()
			]);
		}
	}

	/**
	 * Récupération de l'URL de vignettes de fichiers.
	 *
	 * @return void
	 */
	public static function thumbsItemsSource(): void
	{
		// Vérifications.
		if (!Auth::$isAdmin || !isset($_POST['items_id']) || !is_array($_POST['items_id'])
		|| !isset($_POST['size']) || !preg_match('`^\d{1,4}(x\d{1,4})?$`i', $_POST['size']))
		{
			self::_forbidden();
		}

		$items_id = DB::inInt($_POST['items_id']);
		if (!DB::execute("SELECT * FROM {items} WHERE item_id IN ($items_id)"))
		{
			self::_dbError();
		}

		$thumbs_source = [];
		foreach (DB::fetchAll() as &$i)
		{
			$thumbs_source[$i['item_id']] = App::getThumbSource('item', $i, $_POST['size'], '');
		}

		self::_printResponse(
		[
			'status' => 'success',
			'thumbs_source' => $thumbs_source
		]);
	}



	/**
	 * Vérifie les permissions d'accès à un fichier.
	 *
	 * @param int $item_id
	 *
	 * @return array
	 */
	private static function _itemPermission(int $item_id): array
	{
		$sql = "SELECT i.*
				  FROM {items} AS i
			 LEFT JOIN {categories} AS cat USING (cat_id)
				 WHERE item_id = ?
				   AND item_status = '1'
				   AND %s
				   AND %s";
		$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
		if (!DB::execute($sql, $item_id))
		{
			self::_dbError();
		}
		if (!$infos = DB::fetchRow())
		{
			self::_forbidden();
		}

		return $infos;
	}
}
?>