<?php
declare(strict_types = 1);

/**
 * Initialisation de l'application.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */

// Vérification de la version de PHP.
if (!function_exists('version_compare')
 || !defined('PHP_VERSION')
 || !version_compare(PHP_VERSION, '7.2', '>='))
{
	die('PHP 7.2 or greater is required.');
}

// Gestion du temps d'exécution.
define('START_TIME', microtime(TRUE));
if (function_exists('ignore_user_abort'))
{
	ignore_user_abort(TRUE);
}
if (function_exists('set_time_limit'))
{
	set_time_limit(30);
}

// Constantes de la galerie.
define('GALLERY_ROOT', dirname(__DIR__));
define('GALLERY_HTTPS', (isset($_SERVER['HTTPS'])
		&& in_array(strtolower($_SERVER['HTTPS']), ['on', '1']))
	|| (isset($_SERVER['HTTP_X_FORWARDED_PROTO'])
		&& $_SERVER['HTTP_X_FORWARDED_PROTO'] == 'https')
	|| (isset($_SERVER['HTTP_X_FORWARDED_SSL'])
		&& in_array(strtolower($_SERVER['HTTP_X_FORWARDED_SSL']), ['on', '1']))
	|| $_SERVER['SERVER_PORT'] == 443);
define('GALLERY_HOST', (GALLERY_HTTPS ? 'https://' : 'http://') . $_SERVER['HTTP_HOST']);

// Chargement du fichier de configuration.
if (file_exists(GALLERY_ROOT . '/config/conf.php'))
{
	require_once(GALLERY_ROOT . '/config/conf.php');
}
else
{
	require_once(GALLERY_ROOT . '/config/conf_default.php');
}

// Chargement automatique des classes du noyau.
spl_autoload_register(function(string $class)
{
	$file = GALLERY_ROOT . '/includes/core/' . $class . '.class.php';
	if (!file_exists($file))
	{
		$message = 'Failed opening required file (' . $class . '.class.php).';
		trigger_error($message, E_USER_WARNING);
		die($message);
	}
	require_once($file);
});

// Gestion des erreurs et de la mémoire.
error_reporting(-1);
if (function_exists('ini_set'))
{
	ini_set('memory_limit', '512M');
	ini_set('display_errors', CONF_ERRORS_DISPLAY ? '1' : '0');
}
set_error_handler(['ErrorHandler', 'phpError']);
set_exception_handler(['ErrorHandler', 'exception']);
register_shutdown_function(['ErrorHandler', 'shutdown']);
libxml_use_internal_errors(TRUE);

// Supression des paramètres GET non utilisés.
$_REQUEST = [];
$gets = isset($gets) ? $gets : ['q'];
foreach ($_GET as $name => &$value)
{
	if (!in_array($name, $gets))
	{
		unset($_GET[$name]);
	}
}

// Jeu de caractères pour les fonctions mbstring.
if (!extension_loaded('mbstring'))
{
	die('mbstring not loaded.');
}
mb_internal_encoding(CONF_CHARSET);
if (function_exists('mb_regex_encoding'))
{
	mb_regex_encoding(CONF_CHARSET);
}

// En-têtes HTTP et gestion des cookies.
Auth::sessionHandler();
header('Content-Type: text/html; charset=' . CONF_CHARSET);

// Gestion du cache.
header('Expires: Wed, 21 Oct 2015 07:28:00 GMT');
header('Cache-Control: no-store, no-cache, must-revalidate');
header('Cache-Control: post-check=0, pre-check=0', FALSE);
header('Pragma: no-cache');

// Vérification du jeton anti-CSRF.
Auth::antiCSRFToken();



/**
 * Localisation.
 *
 * @param string $str
 *
 * @return string
 */
function __(string $str): string
{
	if (isset(L10N::$text[$str]))
	{
		L10N::$text[$str] = trim(L10N::$text[$str]);
		return L10N::$text[$str] == '' ? $str : L10N::$text[$str];
	}
	elseif (CONF_DEBUG_MODE)
	{
		trigger_error("Localization not found: \"$str\"", E_USER_NOTICE);
	}
	return $str;
}

/**
 * Console de débogage.
 *
 * @param mixed $data
 *
 * @return array
 */
function l($data = NULL): array
{
	static $log = [];

	$log[] = ['trace' => debug_backtrace(), 'data' => $data];
	return $log;
}
?>