<?php
declare(strict_types = 1);

/**
 * Mise à jour de l'application.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */
require_once(__DIR__ . '/includes/prepend.php');

// Initialisation.
Template::init();
Upgrade::init($latest_version[0] ?? System::APP_VERSION);
if (Template::$data['upgrade_tpl'])
{
	Template::start();
	$tpl =& Template::$data;
}



/**
 * Opérations de mise à jour de l'application.
 */
class Upgrade
{
	/**
	 * La mise à jour a-t-elle réussie ?
	 *
	 * @var bool
	 */
	public static $success = FALSE;



	/**
	 * Paramètres de la table "config" à mettre à jour.
	 *
	 * @var array
	 */
	private static $_config;



	/**
	 * Initialisation.
	 *
	 * @param string $latest_version
	 *
	 * @return void
	 */
	public static function init(string $latest_version): void
	{
		Template::set('upgrade', TRUE);
		Template::set('upgrade_success', FALSE);
		Template::set('upgrade_tpl', FALSE);
		Template::set('upgrade_version', $latest_version);

		// Si le script n'est pas appelé depuis l'interface d'administration,
		// on crée une connexion à la base de données.
		if (DB::$PDO === NULL)
		{
			Template::set('upgrade_tpl', TRUE);

			// Connexion à la base de données.
			if (!DB::connect())
			{
				die('Impossible de se connecter à la base de données.'
					. '<br>Unable to connect to the database.');
			}

			// Récupération de la configuration.
			Config::getDBParams();

			// Authentification utilisateur.
			Auth::cookie();
			L10N::locale();

			// L'application doit-elle être mise à jour ?
			if (Config::$params['app_version'] == System::APP_VERSION)
			{
				Template::set('upgrade', FALSE);
				return;
			}
		}

		// Mise à jour.
		if (!Template::$data['upgrade_tpl'] || !empty($_POST['upgrade']))
		{
			self::_upgrade($latest_version);
		}
	}



	/**
	 * Mise à jour.
	 *
	 * @param string $latest_version
	 *
	 * @return void
	 */
	public static function _upgrade(string $latest_version): void
	{
		self::$_config = Config::$params;

		// Nouveaux paramètres de configuration.
		$config_default = Config::getDBParamDefault();
		foreach ($config_default as $param => &$value)
		{
			if (!array_key_exists($param, self::$_config))
			{
				self::$_config[$param] = $value;
			}
		}

		// Mises à jour vers la version courante.
		

		// Modification de la configuration en base de données.
		if (is_array(self::$_config['app_history']))
		{
			self::$_config['app_history'][] =
			[
				'date' => date('Y-m-d H:i:s'),
				'version' => $latest_version
			];
		}
		else
		{
			self::$_config['app_history'] =
			[
				[
					'date' => date('Y-m-d H:i:s'),
					'version' => $latest_version
				]
			];
		}
		self::$_config['app_version'] = $latest_version;

		// Mise à jour de la table "config".
		ksort(self::$_config);
		$config_values = [];
		$config_params = [];
		foreach (self::$_config as $k => &$v)
		{
			if (is_array($v))
			{
				$v = Utility::jsonEncode($v);
			}
			$config_values[] = "('$k', ?)";
			$config_params[] = $v;
		}
		$config_values = implode(", ", $config_values);

		// Début de la transaction.
		if (!DB::beginTransaction())
		{
			return;
		}

		// Suppression du contenu de la table.
		// DELETE et non TRUNCATE car transaction !
		if (!DB::execute("DELETE FROM {config}"))
		{
			return;
		}

		// Insertion de la nouvelle configuration.
		$sql = "INSERT INTO {config} (conf_name, conf_value) VALUES $config_values";
		if (!DB::execute($sql, $config_params))
		{
			return;
		}

		// Exécution de la transaction.
		if (!DB::commitTransaction())
		{
			return;
		}

		self::$success = TRUE;
		Template::set('upgrade_success', TRUE);
	}
}
if (!Template::$data['upgrade_tpl'])
{
	return;
}
?>
<!DOCTYPE html>
<html lang="<?php echo Auth::$lang; ?>">

<head>
	<title><?php echo __('Mise à jour'); ?> - <?php echo $tpl['app']['name']; ?></title>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width">
	<style type="text/css">
	h1,p,form,body,html
	{
		border: 0;
		margin: 0;
		padding: 0;
	}
	pre
	{
		font-family: "Courier New", serif;
		white-space: pre-wrap;
	}
	html
	{
		font-size: 100%;
	}
	body
	{
		color: black;
		font-family: Verdana, Arial, Helvetica, sans-serif;
		font-size: .8em;
		background: #fafafa;
		padding: 40px 15px;
	}
	a {
		color: black;
		text-decoration: none;
		border-bottom: 1px solid #884C2E;
		position: relative;
	}
	a:hover
	{
		border-width: 2px;
	}
	h1
	{
		font-size: 275%;
		font-weight: normal;
		border-bottom: 3px solid #8A7666;
		padding: 20px 15px 5px 30px;
		color: #555;
		font-family: Georgia, "Times New Roman", serif;
		letter-spacing: .02em;
		background: #F5F2EB url(admin/template/default/style/default/background.png);
	}
	p
	{
		margin-bottom: 15px;
	}
	strong
	{
		font-size: 110%;
	}
	#upgrade
	{
		border: 1px solid #808080;
		max-width: 520px;
		margin: 0 auto;
		text-align: left;
		background: white;
		box-shadow: 0 0 14px #989898;
	}
	#content
	{
		padding: 35px 20px 1px;
		line-height: 1.6em;
	}
	footer
	{
		text-align: center;
		border-top: 2px solid #8A7666;
		background: #F5F2EB url(admin/template/default/style/default/background.png);
		padding: 8px 0;
		margin-top: 40px;
	}
	.icon
	{
		padding: 1px 0 0 26px;
		background-repeat: no-repeat;
		background-position: 0 1px;
		min-height: 19px;
	}
	.icon span
	{
		font-weight: bold;
		color: #333;
	}
	.icon_ok
	{
		background-image: url(admin/template/default/style/default/icons/18x18/success.png);
	}
	.icon_error
	{
		background-image: url(admin/template/default/style/default/icons/18x18/error.png);
	}
	.icon_information
	{
		background-image: url(admin/template/default/style/default/icons/18x18/information.png);
	}
	input[type="submit"]
	{
		border: 1px solid silver;
		margin: 20px 0 0;
		padding: 5px 8px;
		background: linear-gradient(0deg, #E5E5E5, #FFFFFF);
		white-space: nowrap;
		border-radius: 5px;
		outline: none;
		font-size: 120%;
	}
	input[type="submit"]:active
	{
		background: linear-gradient(0deg, #FFFFFF, #E5E5E5);
	}
	input[type="submit"]:focus
	{
		border-color: gray;
	}
	input[type="submit"]:hover
	{
		box-shadow: 0 0 4px silver;
	}
	</style>
</head>

<body>

	<div id="upgrade">
		<h1><?php echo __('Mise à jour'); ?></h1>
		<div id="content">
<?php if (!$tpl['upgrade']) : ?>
			<p class="icon icon_information"><span><?php echo __('iGalerie est déjà à jour.'); ?></span></p>
<?php elseif (empty($_POST['upgrade'])) : ?>
			<p><?php printf(__('Cliquez sur ce bouton pour mettre à jour l\'application vers la version %s.'), '<br><strong>' . $tpl['upgrade_version'] . '</strong>'); ?></p>
			<form action="" method="post">
				<div>
					<input name="anticsrf" value="<?php echo $tpl['anticsrf'](); ?>" type="hidden">
					<input name="upgrade" value="<?php echo __('Démarrer la mise à jour'); ?>" type="submit">
				</div>
			</form>
<?php elseif ($tpl['upgrade_success']) : ?>
			<p class="icon icon_ok"><span><?php echo __('Mise à jour effectuée avec succès !'); ?></span></p>
			<p><?php printf(__('N\'oubliez pas de supprimer le fichier %s avant de poursuivre.'), basename(__FILE__)); ?></p>
<?php else : ?>
			<p class="icon icon_error"><span><?php echo __('La mise à jour a échouée.'); ?></span></p>
<?php endif; ?>
		</div>
		<footer><?php printf(__('Propulsé par %s'), '<a class="ex" href="' . $tpl['app']['website'] . '">' . $tpl['app']['name'] . '</a>'); ?></footer>
	</div>

	<?php $tpl['print']('logs'); ?>

	<?php $tpl['print']('errors'); ?>

</body>

</html>